@@ -69,19 +69,14 @@ from . import get_logger
 from .consts import (
     DRAFT,
     MAX_CHUNK_SIZE,
+    RETRY_STATUSES,
     DandiInstance,
     known_instances,
     known_instances_rev,
 )
 from .exceptions import NotFoundError, SchemaVersionError
 from .keyring import keyring_lookup
-from .utils import (
-    USER_AGENT,
-    check_dandi_version,
-    ensure_datetime,
-    is_interactive,
-    try_multiple,
-)
+from .utils import USER_AGENT, check_dandi_version, ensure_datetime, is_interactive
 
 lgr = get_logger()
 
@@ -117,7 +112,7 @@ class RESTFullAPIClient:
         json=None,
         headers=None,
         json_resp=True,
-        retry=None,
+        retry_statuses=(),
         **kwargs,
     ):
         """
@@ -156,13 +151,10 @@ class RESTFullAPIClient:
                 print(resp.headers)  # Dict of headers
 
         :type json_resp: bool
-        :param retry: an optional tenacity `retry` argument for retrying the
-            request method
+        :param retry_statuses: a sequence of HTTP response status codes to
+            retry in addition to `dandi.consts.RETRY_STATUSES`
         """
 
-        # Look up the HTTP method we need
-        f = getattr(self.session, method.lower())
-
         url = self.get_url(path)
 
         if headers is None:
@@ -172,30 +164,49 @@ class RESTFullAPIClient:
 
         lgr.debug("%s %s", method.upper(), url)
 
-        # urllib3's ConnectionPool isn't thread-safe, so we sometimes hit
-        # ConnectionErrors on the start of an upload.  Retry when this happens.
-        # Cf. <https://github.com/urllib3/urllib3/issues/951>.
-        doretry = tenacity.retry_if_exception_type(
-            requests.ConnectionError
-        ) | tenacity.retry_if_result(lambda r: r.status_code == 503)
-        if retry is not None:
-            doretry |= retry
-
         try:
-            result = try_multiple(12, doretry, 1.25)(
-                f,
-                url,
-                params=params,
-                data=data,
-                files=files,
-                json=json,
-                headers=headers,
-                **kwargs,
-            )
+            for i, attempt in enumerate(
+                tenacity.Retrying(
+                    wait=tenacity.wait_exponential(exp_base=1.25, multiplier=1.25),
+                    # urllib3's ConnectionPool isn't thread-safe, so we
+                    # sometimes hit ConnectionErrors on the start of an upload.
+                    # Retry when this happens.
+                    # Cf. <https://github.com/urllib3/urllib3/issues/951>.
+                    retry=tenacity.retry_if_exception_type(
+                        (requests.ConnectionError, requests.HTTPError)
+                    ),
+                    stop=tenacity.stop_after_attempt(12),
+                    reraise=True,
+                )
+            ):
+                with attempt:
+                    if attempt.retry_state.attempt_number > 1:
+                        lgr.warning("Retrying %s %s", method.upper(), url)
+                    result = self.session.request(
+                        method,
+                        url,
+                        params=params,
+                        data=data,
+                        files=files,
+                        json=json,
+                        headers=headers,
+                        **kwargs,
+                    )
+                    if result.status_code in [*RETRY_STATUSES, *retry_statuses]:
+                        result.raise_for_status()
         except Exception:
             lgr.exception("HTTP connection failed")
             raise
 
+        if i > 0:
+            lgr.info(
+                "%s %s succeeded after %d retr%s",
+                method.upper(),
+                url,
+                i,
+                "y" if i == 1 else "ies",
+            )
+
         lgr.debug("Response: %d", result.status_code)
 
         # If success, return the json object. Otherwise throw an exception.
@@ -552,18 +563,6 @@ class RemoteDandiset:
             **data,
         )
 
-    def _mkasset_from_metadata(self, metadata: Dict[str, Any]) -> "RemoteAsset":
-        return RemoteAsset(
-            client=self.client,
-            dandiset_id=self.identifier,
-            version_id=self.version_id,
-            identifier=metadata["identifier"],
-            path=metadata["path"],
-            size=metadata["contentSize"],
-            modified=metadata["dateModified"],
-            _metadata=metadata,
-        )
-
     def json_dict(self) -> Dict[str, Any]:
         """
         Convert to a JSONable `dict`, omitting the ``client`` attribute and
@@ -660,14 +659,14 @@ class RemoteDandiset:
             json={"metadata": metadata, "name": metadata.get("name", "")},
         )
 
-    def wait_until_valid(self, min_time=20):
+    def wait_until_valid(self, max_time=120):
         """
         Wait for a Dandiset to be valid.  Validation is a background celery
         task which runs asynchronously, so we need to wait for it to complete.
         """
         lgr.debug("Waiting for Dandiset %s to complete validation ...", self.identifier)
         start = time()
-        while time() - start < min_time:
+        while time() - start < max_time:
             r = self.client.get(f"{self.version_api_path}info/")
             if "status" not in r:
                 # Running against older version of dandi-api that doesn't
@@ -706,9 +705,10 @@ class RemoteDandiset:
         ID.  If the given asset does not exist, a `requests.HTTPError` is
         raised with a 404 status code.
         """
-        return self._mkasset_from_metadata(
-            self.client.get(f"{self.version_api_path}assets/{asset_id}/")
-        )
+        metadata = self.client.get(f"{self.version_api_path}assets/{asset_id}/")
+        asset = self.get_asset_by_path(metadata["path"])
+        asset._metadata = metadata
+        return asset
 
     def get_assets_with_path_prefix(self, path: str) -> Iterator["RemoteAsset"]:
         """
@@ -951,7 +951,6 @@ class BaseRemoteAsset(APIBase):
     identifier: str = Field(alias="asset_id")
     path: str
     size: int
-    modified: datetime
     #: Metadata supplied at initialization; returned when metadata is requested
     #: instead of performing an API call
     _metadata: Optional[Dict[str, Any]] = PrivateAttr(default_factory=None)
@@ -974,7 +973,6 @@ class BaseRemoteAsset(APIBase):
             identifier=metadata["identifier"],
             path=metadata["path"],
             size=metadata["contentSize"],
-            modified=metadata["dateModified"],
             _metadata=metadata,
         )
 
@@ -1116,6 +1114,10 @@ class RemoteAsset(BaseRemoteAsset):
     #: The identifier for the version of the Dandiset to which the asset
     #: belongs
     version_id: str
+    #: The date at which the asset was created
+    created: datetime
+    #: The date at which the asset was last modified
+    modified: datetime
 
     @property
     def api_path(self) -> str:
@@ -1186,7 +1188,7 @@ def _upload_part(storage_session, fp, lock, etagger, asset_path, part):
         part["upload_url"],
         data=chunk,
         json_resp=False,
-        retry=tenacity.retry_if_result(lambda r: r.status_code == 500),
+        retry_statuses=[500],
     )
     server_etag = r.headers["ETag"].strip('"')
     lgr.debug(
